function [dfds dqds] = dFdQ( iG, stress, Fi, cohesion, Psi )
%obliczenia pochodnych funkcji kryterium i potencjalu plastycznego
%wg mdcpl.f95, Smith&Griffiths
                
    sx=stress(1);
    sy=stress(2);
    txy=stress(3); 
    sz=stress(4);   
    zero=0; one=1; two=2; d3=3; d4=4; d6=6; sq3=sqrt(3);
    snph=sin(Fi);
    snps=sin(Psi);
        
    t3=one/d3;
    t4=(sx+sy+sz)*t3;
    t8=sz-t4;
    t10=txy^2;
    t16=(sx-sy)^2;
    t18=(sy-sz)^2;
    t20=(sz-sx)^2;
    t25=sqrt((t16+t18+t20)/d6+t10);
    t26=t25^2;
    t30=d3*sq3*((sx-t4)*(sy-t4)*t8-t8*t10)/two/t26/t25;
    if t30 > one 
        t30=one;
    end
   if t30 < -one 
       t30=-one;
   end
   t31=asin(t30);
   t33=sin(t31*t3);
   snth=-t33;
   
   %Drucker-Prager
   if abs(snth) > 0.49
        pm=-one;
        if snth < zero 
            pm=one;
        end
        t2=snph/d3;
        t4=(sx-sy)^2;
        t6=(sy-sz)^2;
        t8=(sz-sx)^2;
        t10=one/d6;
        t12=txy^2;
        t14=sqrt((t4+t6+t8)*t10+t12);
        t17=one/t14/sq3;
        t18=one/two;
        t19=t17*t18;
        t21=d3+pm*snph;
        t23=two*sy;
        t24=two*sz;
        t31=two*sx;
        dfds(1,1)=t2+t19*t21*(d4*sx-t23-t24)*t10/two;
        dfds(2,1)=t2+t19*t21*(-t31+d4*sy-t24)*t10/two;
        dfds(3,1)=t17*t18*t21*txy;
        dfds(4,1)=t2+t19*t21*(-t23+d4*sz-t31)*t10/two;
        t2=snps/d3;
        t21=d3+pm*snps;
        dqds(1,1)=t2+t19*t21*(d4*sx-t23-t24)*t10/two;
        dqds(2,1)=t2+t19*t21*(-t31+d4*sy-t24)*t10/two;
        dqds(3,1)=t17*t18*t21*txy;
        dqds(4,1)=t2+t19*t21*(-t23+d4*sz-t31)*t10/two;
   %Coulomb-Mohr
   else
        t1=one/d3;
        t2=snph*t1;
        t4=(sx-sy)^2;
        t6=(sy-sz)^2;
        t8=(sz-sx)^2;
        t10=one/d6;
        t12=txy^2;
        t13=(t4+t6+t8)*t10+t12;
        t14=sqrt(t13);
        t16=d3*sq3;
        t18=(sx+sy+sz)*t1;
        t19=sx-t18;
        t20=sy-t18;
        t21=t19*t20;
        t22=sz-t18;
        t25=t21*t22-t22*t12;
        t26=one/two;
        t28=t14^2;
        t30=one/t28/t14;
        t31=t16*t25*t26*t30;
        if t31 > one 
            t31=one;
        end
        if t31 < -one
            t31=-one;
        end
        t33=asin(t31);
        t34=t33*t1;
        t35=cos(t34);
        t36=sin(t34);
        t38=one/sq3;
        t41=one/t14*(t35+t36*snph*t38);
        t43=two*sy;
        t44=two*sz;
        t46=(d4*sx-t43-t44)*t10;
        t49=one-t1;
        t53=t19*t1*t22;
        t54=t21*t1;
        t55=t1*t12;
        t60=t16*t25;
        t61=t28^2;
        t64=t26/t61/t14;
        t68=t16*(t49*t20*t22-t53-t54+t55)*t26*t30-d3/two*t60*t64*t46;
        t70=d3^2;
        t71=sq3^2;
        t73=t25^2;
        t74=two^2;
        t77=t13^2;
        t83=sqrt(one-t70*t71*t73/t74/t77/t13);
        t84=one/t83;
        t85=t84*t1;
        t89=t2*t38;
        t94=two*sx;
        t97=(-t94+d4*sy-t44)*t10;
        t101=t1*t20*t22;
        t111=t16*(-t101+t19*t49*t22-t54+t55)*t26*t30-d3/two*t60*t64*t97;
        t129=-two*t16*t22*txy*t26*t30-d3*t60*t64*txy;
        t140=(-t43+d4*sz-t94)*t10;
        t152=t16*(-t101-t53+t21*t49-t49*t12)*t26*t30-d3/two*t60*t64*t140;
        dfds(1,1)=t2+t41*t46/two+t14*(-t36*t68*t85+t35*t68*t84*t89);
        dfds(2,1)=t2+t41*t97/two+t14*(-t36*t111*t85+t35*t111*t84*t89);
        dfds(3,1)=t41*txy+t14*(-t36*t129*t85+t35*t129*t84*t89);
        dfds(4,1)=t2+t41*t140/two+t14*(-t36*t152*t85+t35*t152*t84*t89);
        t2=snps*t1;
        t41=one/t14*(t35+t36*snps*t38);
        t89=t2*t38;
        dqds(1,1)=t2+t41*t46/two+t14*(-t36*t68*t85+t35*t68*t84*t89);
        dqds(2,1)=t2+t41*t97/two+t14*(-t36*t111*t85+t35*t111*t84*t89);
        dqds(3,1)=t41*txy+t14*(-t36*t129*t85+t35*t129*t84*t89);
        dqds(4,1)=t2+t41*t140/two+t14*(-t36*t152*t85+t35*t152*t84*t89);   
   end
   
end
 